%{
    #define _SKIP_YYFLEXLEXER_
    #include "scanner.ih"
    #include "parser.h"

    #include "preinclude.h"
	#include "tokens.h"
%}

%option yyclass="Scanner" outfile="scanner.cc"
%option c++ 8bit warn noyywrap yylineno
%option debug
%option case-insensitive

%x ST_STRING
%x ST_COMMENT
%x ST_ML_COMMENT

DIGIT [0-9]
HEXDIGIT [0-9a-f]
SYMBOL [_a-z][_a-z0-9]*

HEX "0x"({HEXDIGIT})+
DEC ({DIGIT})+


WS [ \t\r\n]
NL [\n]
NON_NL [^\r\n]

%%

<INITIAL>\"	{ 
		BEGIN(ST_STRING);
		quoted_str = "";
	}

<INITIAL>\/\/	{ 
		BEGIN(ST_COMMENT);
	}

<INITIAL>\/\*	{ 
		BEGIN(ST_ML_COMMENT);
	}

<ST_ML_COMMENT>\*\/	{ 
		BEGIN(INITIAL);
	}

<ST_COMMENT>{NL} {
		BEGIN(INITIAL);
	}

<ST_COMMENT>. {}
<ST_ML_COMMENT>. {}

<ST_STRING>{
	\\. 	{quoted_str += yytext+1;}
	[^\\"]	{quoted_str += yytext;}
	\" {
			BEGIN(INITIAL);
			loc->line = yylineno; 
			TokenText *t;
			try {
				t = new TokenText(quoted_str, true);
			} catch (char const * m) {
				throw CompilerException(m, loc->filename, yylineno);
			}
			t->line(yylineno);
			t->filename(loc->filename);
			*d_semval = Semantic(t);
			return Parser::T_QUOTED_STRING;
		}
}

<INITIAL>{

	'\\r' {
			loc->line = yylineno;
			TokenInt *i = new TokenInt('\r');
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		   }
	'\\n' {
			loc->line = yylineno;
			TokenInt *i = new TokenInt('\n');
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		   }
	'\\t' {
			loc->line = yylineno;
			TokenInt *i = new TokenInt('\t');
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		   }
	'\\v' {
			loc->line = yylineno;
			TokenInt *i = new TokenInt('\v');
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		   }
	'\\\\' {
			loc->line = yylineno;
			TokenInt *i = new TokenInt('\\');
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		   }
	'{NON_NL}' {
			loc->line = yylineno;
			TokenInt *i = new TokenInt(*(yytext+1));
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		   }


	"#INCLUDE" {loc->line = yylineno; return Parser::T_INCLUDE;}
	"CONFIG" {loc->line = yylineno; return Parser::T_CONFIG;}
	"CONST" {loc->line = yylineno; return Parser::T_CONST;}
	"ELSE" {loc->line = yylineno; return Parser::T_ELSE;}
	"EVENT" {loc->line = yylineno; return Parser::T_EVENT;}
	"FROM" {loc->line = yylineno; return Parser::T_FROM;}
	"FUNCTION" {loc->line = yylineno; return Parser::T_FUNCTION;}
	"IF" {loc->line = yylineno; return Parser::T_IF;}
	"INT" {loc->line = yylineno; return Parser::T_INT;}
	"REPEAT" {loc->line = yylineno; return Parser::T_REPEAT;}
	"RETURN" {loc->line = yylineno; return Parser::T_RETURN;}
	"STRING" {loc->line = yylineno; return Parser::T_STRING;}
	"TO" {loc->line = yylineno; return Parser::T_TO;}
	"UNTIL" {loc->line = yylineno; return Parser::T_UNTIL;}
	"WHILE" {loc->line = yylineno; return Parser::T_WHILE;}


	{DEC} {loc->line = yylineno; 
			TokenInt *i = new TokenInt(yytext);
			i->line(yylineno);
			i->filename(loc->filename);
			*d_semval = Semantic(i);
			return Parser::T_NUMBER;
		}

		/*
	{HEX} { 
			yylval = alloc_node(NT_NUMBER, yylineno);
			yylval->data.intval = strtol(yytext, 0, 16);
			return T_NUMBER;
		}
		*/
	
	[a-z_]([a-z_0-9])* {
			loc->line = yylineno; 
			TokenText *r = new TokenText(yytext);
			r->line(yylineno);
			r->filename(loc->filename);
			*d_semval = Semantic(r);
			return Parser::T_IDENTIFIER;
		}


	">=" {loc->line = yylineno; return Parser::T_GE;}
	"<=" {loc->line = yylineno; return Parser::T_LE;}
	"!=" {loc->line = yylineno; return Parser::T_NE;}
	"&&" {loc->line = yylineno; return Parser::T_L_AND;}
	"||" {loc->line = yylineno; return Parser::T_L_OR;}

	{WS} {}
	.  {loc->line = yylineno; return *yytext;}

	<<EOF>> {
		if (d_level) return Parser::T_EOF;
		else yyterminate();
	}
}

%%



