%class-name Parser

%filenames parser
%parsefun-source parser.cc
%scanner scanner.h

//%debug
%error-verbose
%baseclass-preinclude preinclude.h

/*
%locationstruct {
	int line;
}
*/
%ltype ParserLocation


%expect 1

%stype   Semantic

%token T_CONFIG
%token T_CONST
%token T_IDENTIFIER
%token T_ELSE
%token T_EVENT
%token T_FROM
%token T_FUNCTION
%token T_IF
%token T_INT
%token T_NUMBER
%token T_QUOTED_STRING
%token T_REPEAT
%token T_RETURN
%token T_STRING
%token T_TO
%token T_UNTIL
%token T_WHILE
%token T_INCLUDE
%token T_EOF
//%token T_LE
//%token T_GE
//%token T_NE
//%token T_L_AND
//%token T_L_OR

%left T_L_OR
%left T_L_AND
%left '|'
%left '^'
%left '&'
%left '=' T_NE
%left '<' '>' T_LE T_GE
%left '+' '-'
%left '*' '/'


%expect 1
%lines 


%%

program : sections
		;

sections: section
		| sections section;

section: include_section
	   | constant_section
	   | var_section
	   | config_section
	   | event_section
	   | func_section
	   | possible_include_end
	   ;


possible_include_end: T_EOF {
						d_scanner.leaveInclude();
						d_scanner.yy_delete_buffer(d_scanner.currentBuffer());
						d_scanner.yy_delete_buffer(d_scanner.currentBuffer());
						--include_stack_ptr;
						d_scanner.yy_switch_to_buffer(include_stack[include_stack_ptr].buf_state);
						d_scanner.setLineNo(include_stack[include_stack_ptr].lineno);
						delete d_loc__.filename;
						d_loc__.filename =  include_stack[include_stack_ptr].filename;
                 }
				 ;


include_section: T_INCLUDE T_QUOTED_STRING {
						string name = $2.downcast<TokenText>()->text();
						string incName = findInclude(name);
						if (incName == "") {
							throw CompilerException(name + ": no such file", d_loc__.filename, $2.base()->line());
						}
						ifstream *file = new ifstream();
						file->open(incName.c_str());
						if (!file->is_open()) {
							throw CompilerException("Cannot open " + incName, d_loc__.filename, $2.base()->line());
						}
						if (include_stack_ptr >= MAX_INCLUDE_DEPTH) {
							throw CompilerException("Includes nested too deeply", d_loc__.filename, $2.base()->line());
						}

						include_stack[include_stack_ptr].buf_state = d_scanner.currentBuffer();
						include_stack[include_stack_ptr].lineno = $2.base()->line();;
						include_stack[include_stack_ptr].filename = d_loc__.filename;
						d_loc__.filename = new string(incName);
						include_stack_ptr++;
						yy_buffer_state* buf =  d_scanner.yy_create_buffer(file, 16384 );
						d_scanner.yy_switch_to_buffer(buf);
						d_scanner.enterInclude();
						d_scanner.setLineNo(1);
	
	/*
	void yy_switch_to_buffer( struct yy_buffer_state* new_buffer );
	struct yy_buffer_state* yy_create_buffer( FLEX_STD istream* s, int size );
	void yy_delete_buffer( struct yy_buffer_state* b );
	void yyrestart( FLEX_STD istream* s );

	void yypush_buffer_state( struct yy_buffer_state* new_buffer );
	void yypop_buffer_state();

	virtual int yylex();
	virtual void switch_streams( FLEX_STD istream* new_in, FLEX_STD ostream* new_out = 0 );
	virtual int yywrap();
	*/
					}
				;

constant_section: T_CONST constant_defs ';'
				;

config_section : T_CONFIG constant_defs ';'
			   ;

constant_defs : constant_def
			  | constant_defs ',' constant_def
			  ;

constant_def : numeric_constant
			 | string_constant
			 | list_constant
			 ;

string_constant: T_IDENTIFIER '=' T_QUOTED_STRING {
						try {
							Program::instance()->defineConstant(std::string($1.downcast<TokenText>()->text()), $3);
						} catch (char const * m) {
							throw CompilerException(m, d_loc__.filename, $1.base()->line());
						}
					}
			   ;

numeric_constant: T_IDENTIFIER '=' expression {
						try {
							Program::instance()->defineConstant(std::string($1.downcast<TokenText>()->text()), $3);
						} catch (char const * m) {
							throw CompilerException(m, d_loc__.filename, $1.base()->line());
						}
					}
				;

list_constant: T_IDENTIFIER '=' '[' strings_list ']' {
						TokenList *l = new TokenList(strings_list);
						l->line($1.base()->line());
						l->filename($1.base()->filename());
						try {
							Program::instance()->defineConstant(std::string($1.downcast<TokenText>()->text()), Semantic(l));
						} catch (char const * m) {
							throw CompilerException(m, d_loc__.filename, $1.base()->line());
						}
						strings_list.clear();
					}
				;

strings_list : T_QUOTED_STRING {
					strings_list.push_back($1.downcast<TokenText>()->text());
				}
			 | strings_list ',' T_QUOTED_STRING {
					strings_list.push_back($3.downcast<TokenText>()->text());
				 }
			 ;

var_section : var_defs ';'
			;

var_defs : var_def
		 | var_defs ',' var_def
		 ;

var_def : T_INT T_IDENTIFIER {
				try {
					Program::instance()->defineGlobal(std::string($2.downcast<TokenText>()->text()));
				} catch (char const * m) {
					throw CompilerException(m, d_loc__.filename, $2.base()->line());
				}
			}
		| T_INT T_IDENTIFIER '[' expression ']' {
				try {
					Program::instance()->defineGlobal(std::string($2.downcast<TokenText>()->text()), Global::INT, $4);
				} catch (char const * m) {
					throw CompilerException(m, d_loc__.filename, $2.base()->line());
				}
			}
		| T_STRING T_IDENTIFIER '[' expression ']' {
				try {
					Program::instance()->defineGlobal(std::string($2.downcast<TokenText>()->text()), Global::STRING, $4);
				} catch (char const * m) {
					throw CompilerException(m, d_loc__.filename, $2.base()->line());
				}
			}
		;

func_section : T_FUNCTION func_def  '(' params_defs ')' local_defs statement
				{
					try {
						Program::instance()->defineFunction(
							std::string($2.downcast<TokenText>()->text()),
							params_list,
							locals_list,
							$7
						);
					} catch (char const * m) {
						throw CompilerException(m, d_loc__.filename, $2.base()->line());
					}
					params_list.clear();
					locals_list.clear();
				}
			 ;

event_section : T_EVENT event_def local_defs statement 
				{
					try {
						Program::instance()->defineEvent(
							std::string($2.downcast<TokenText>()->text()),
							locals_list,
							$4
						);
					} catch (char const * m) {
						throw CompilerException(m, d_loc__.filename, $2.base()->line());
					}
					params_list.clear();
					locals_list.clear();
				}
			 ;

func_def : T_IDENTIFIER
		 {
			$$ = $1;
		 }
;

event_def : T_IDENTIFIER
		 {
			$$ = $1;
		 }
;

local_defs : 
			| ':' ne_local_defs
				{$$ = $2;}
			;

ne_local_defs : ne_local_defs ',' locals_def
			   | locals_def
			   ;
locals_def : T_IDENTIFIER {
					std::string n = $1.downcast<TokenText>()->text();
					typedef std::vector<std::string>::iterator it_type;
					it_type i = std::find<it_type, std::string>(params_list.begin(), params_list.end(), n);
					if (i != params_list.end()) {
						throw "Duplicate parameter name";
					}
					locals_list.insert(locals_list.end(), n);
				}
		   ;

params_defs : 
			| ne_params_defs
			;

ne_params_defs : ne_params_defs ',' params_def
			   | params_def
			   ;

params_def : T_IDENTIFIER {
					std::string n = $1.downcast<TokenText>()->text();
					typedef std::vector<std::string>::iterator it_type;
					it_type i = std::find<it_type, std::string>(params_list.begin(), params_list.end(), n);
					if (i != params_list.end()) {
						throw "Duplicate parameter name";
					}
					params_list.insert(params_list.end(), n);
				}
		   ;

func_call : T_IDENTIFIER '('  { push_call(); }  actual_params ')' {
					TokenCall *call = new TokenCall(
						$1.downcast<TokenText>()->text(),
						call_params_list[call_params_list.size()-1]
					);
					call->line($1.base()->line());
					call->filename($1.base()->filename());
					$$ = Semantic(call);
					pop_call();
				}
		  ;

native_call : T_IDENTIFIER '.' T_IDENTIFIER '(' {push_call();}  actual_params ')' {
				TokenCall *call = new TokenCall(
					$1.downcast<TokenText>()->text(),
					$3.downcast<TokenText>()->text(),
					call_params_list[call_params_list.size()-1]
				);
				call->line($1.base()->line());
				call->filename($1.base()->filename());
				$$ = Semantic(call);
				pop_call();
			}
		  ;

actual_params :
			  | actual_params_list
			  ;

actual_params_list : expression {
							 call_params_list[call_params_list.size()-1].push_back($1);
						}
				   | actual_params_list ',' expression {
							call_params_list[call_params_list.size()-1].push_back($3);
						}
				   ;


statement : single_statement
		   | compound_statement
		   ;

compound_statement: '{' statements_list '}' {
						TokenCompound *c = new TokenCompound();
						c->line($2.base()->line());
						c->filename($2.base()->filename());
						c->add($2);
						$$ = Semantic(c);
					}
				  | '{' '}' 
				  ;

statements_list : 
			
		   statements_list single_statement  {
					TokenCompound *c = $1.downcast<TokenCompound>();
					c->add($2);
					$$ = $1;
				}
		   | statements_list compound_statement  {
					TokenCompound *c = $1.downcast<TokenCompound>();
					c->add($2);
					$$ = $1;
				}
			|	single_statement {
						TokenCompound *c = new TokenCompound();
						c->line($1.base()->line());
						c->filename($1.base()->filename());
						c->add($1);
						$$ = Semantic(c);
					}
			| compound_statement {
						TokenCompound *c = new TokenCompound();
						c->line($1.base()->line());
						c->filename($1.base()->filename());
						c->add($1);
						$$ = Semantic(c);
					}
		   ;


single_statement : ';' {
						TokenCompound *c = new TokenCompound();
						c->line($1.base()->line());
						c->filename($1.base()->filename());
						$$ = Semantic(c);
				 }
		  | assignment ';'
		  | control_statement
		  | func_call ';' {
					TokenCall *call = $1.downcast<TokenCall>();
					call->discard_result = true;
					call->line($1.base()->line());
					$$ = $1;
				}
		  | native_call ';'  {
					TokenCall *call = $1.downcast<TokenCall>();
					call->line($1.base()->line());
					call->discard_result = true;
					$$ = $1;
				}
		  | return ';' 
		  ;

assignment : T_IDENTIFIER '=' expression {
					TokenId *var = new TokenId($1.downcast<TokenText>()->text());
					var->line($1.base()->line());
					var->filename($1.base()->filename());
					TokenExpression *exp = new TokenExpression(OP_ASSIGN, var, $3);
					exp->line($3.base()->line());
					exp->filename($3.base()->filename());
					$$ = Semantic(exp);
				}
			| array_ref '=' expression {
					TokenExpression *exp = new TokenExpression(OP_ASSIGN, $1, $3);
					exp->line($1.base()->line());
					exp->filename($1.base()->filename());
					$$ = Semantic(exp);
				}
		   ;

control_statement : if_statement
				  | loop_statement
				  ;

return : T_RETURN {
				TokenRet *r = new TokenRet(NULL);
				r->line(d_loc__.line);
				r->filename(d_loc__.filename);
				$$ = Semantic(r);
			}
	   | T_RETURN expression {
				TokenRet *r = new TokenRet($2);
				r->line(d_loc__.line);
				r->filename(d_loc__.filename);
				$$ = Semantic(r);
			}
	   ;

loop_statement : from_to_loop
			   | while_loop
			   | until_loop
			   ;


from_to_loop : T_REPEAT T_IDENTIFIER T_FROM expression T_TO expression statement {
					TokenRepeat *r = new TokenRepeat($2, $4, $6, $7);
					r->line($1.base()->line());
					r->filename($1.base()->filename());
					$$ = Semantic(r);
				}

			 ;

while_loop : T_WHILE expression statement {
					TokenWhile *r = new TokenWhile($2, $3);
					r->line($1.base()->line());
					r->filename($1.base()->filename());
					$$ = Semantic(r);
				}
		   ;

until_loop : T_UNTIL expression statement  {
					TokenUntil *r = new TokenUntil($2, $3);
					r->line($1.base()->line());
					r->filename($1.base()->filename());
					$$ = Semantic(r);
				}
		   ;

if_statement : if_then
			 | if_then_else
			 ;

if_then : T_IF expression statement {
			TokenIf *r = new TokenIf($2, $3, NULL);
			r->line($2.base()->line());
			r->filename($2.base()->filename());
			$$ = Semantic(r);
		}
		;

if_then_else : T_IF expression  statement T_ELSE statement {
					TokenIf *r = new TokenIf($2, $3, $5);
					r->line($2.base()->line());
					r->filename($2.base()->filename());
					$$ = Semantic(r);
				}
			 ;

array_ref: T_IDENTIFIER  '[' expression ']' {
						TokenArrayref *var = new TokenArrayref($1.downcast<TokenText>()->text(), $3);
						var->line($1.base()->line());
						var->filename($1.base()->filename());
						$$ = Semantic(var);
					}
		;

expression : T_IDENTIFIER {
					TokenId *id = new TokenId($1.downcast<TokenText>()->text());
					id->line($1.base()->line());
					id->filename($1.base()->filename());
					$$ = Semantic(id);
				}
		   | T_NUMBER 
				{
					$$ = $1;

				}
		   | T_QUOTED_STRING {
						string name = Program::instance()->defineAnonymousConstant($1);
						TokenId *id = new TokenId(name);
						id->line($1.base()->line());
						id->filename($1.base()->filename());
						$$ = Semantic(id);
					}
		   | array_ref
		   | expression '+' expression {OPERATOR_EXP($$, OP_PLUS, $1, $3)}
		   | expression '-' expression {OPERATOR_EXP($$, OP_MINUS, $1, $3)}
		   | expression '*' expression {OPERATOR_EXP($$, OP_MUL, $1, $3)}
		   | expression '/' expression {OPERATOR_EXP($$, OP_DIV, $1, $3)}
		   | expression '&' expression {OPERATOR_EXP($$, OP_AND, $1, $3)}
		   | expression '|' expression {OPERATOR_EXP($$, OP_OR, $1, $3)}
		   | expression '^' expression {OPERATOR_EXP($$, OP_XOR, $1, $3)}
		   | expression '>' expression {OPERATOR_EXP($$, OP_GT, $1, $3)}
		   | expression '<' expression {OPERATOR_EXP($$, OP_LT, $1, $3)}
		   | expression '=' expression {OPERATOR_EXP($$, OP_EQ, $1, $3)}
		   | expression T_NE expression {OPERATOR_EXP($$, OP_NE, $1, $3)}
		   | expression T_GE expression {OPERATOR_EXP($$, OP_GE, $1, $3)}
		   | expression T_LE expression {OPERATOR_EXP($$, OP_LE, $1, $3)}
		   | expression T_L_AND expression {OPERATOR_EXP($$, OP_L_AND, $1, $3)}
		   | expression T_L_OR expression {OPERATOR_EXP($$, OP_L_OR, $1, $3)}
		   | '!' expression {OPERATOR_EXP($$, OP_NOT, $2, NULL)}
		   | '(' expression ')' {$$ = $2;}
		   | func_call
		   | native_call

		   ;

