%class-name NativeDescParser

%filenames nativedescparser
%parsefun-source nativedescparser.cc
%scanner nativedescscanner.h

//%debug
//%error-verbose
//%baseclass-preinclude preinclude.h

%stype  std::string

%lsp-needed 


%token T_EVENTS
%token T_NATIVE
%token T_ID
%token T_INT
%token T_STRING
%token T_ARRAY
%token T_LIST
%token T_NUMBER
%token T_RAM
%token T_PROGRAM

%lines 

%%

description: sections
		   ;

sections : section
		 | sections section
		 ;

section: events_section
	   | methods_section
	   | ram_section
	   | program_section
	   ;

ram_section : T_RAM ':' T_NUMBER ';' {
					d_result.ram = atoi($3.c_str());
				}
		    ;

program_section : T_PROGRAM ':' T_NUMBER ';' {
					d_result.program = atoi($3.c_str());
				}
		    ;

events_section: T_EVENTS '{' event_descriptions '}'
			  ;

event_descriptions: event_description
				  | event_descriptions event_description
				  ;

event_description: T_ID  ';' {
						string str = string($1);
						std::transform(str.begin(), str.end(), str.begin(), ::tolower);
						d_result.events.push_back(str);
					}
				 ;

methods_section: T_NATIVE '{' object_descriptions '}'
			   ;

object_descriptions : object_description
					| object_descriptions object_description
					;

object_description : T_ID '{' method_descriptions '}' {
							unsigned int i;
							for (i = 0; i < functions.size(); i++) {
								functions[i].name = string($1) + '.' + functions[i].name;
								std::transform(functions[i].name.begin(), functions[i].name.end(), functions[i].name.begin(), ::tolower);
								d_result.functions.push_back(functions[i]);
							}
							functions.clear();
						}
					;

method_descriptions : method_description
					| method_descriptions method_description
					;

method_description: T_ID '(' params_list ')' ';' {
						func.name = $1;
						functions.push_back(func);
						func.params.clear();
					}
				  ;

params_list : 
			| ne_params_list;

ne_params_list: param_def
			  | ne_params_list ',' param_def
			  ;

param_def : T_STRING { func.params.push_back(Base::STRING); }
		  | T_INT { func.params.push_back(Base::INT); }
		  | T_ARRAY{ func.params.push_back(Base::ARRAY);}
		  | T_LIST{ func.params.push_back(Base::LIST); }
		  ;

